package org.textube;

import java.io.File;
import java.io.IOException;
import java.net.URLDecoder;

import org.textube.util.Configuration;

/**
 * Boot to handle necessary arguments.
 */
public class Boot {
	
	/**
	 * Boot main method.
	 */
	public static void main(final String[] args) throws IOException {
		if (new Throwable().getStackTrace().length > 1) {
			throw new IOException();
		}
		Configuration.mkdirs();
		Configuration.mklibs();
		String location = Boot.class.getProtectionDomain().getCodeSource().getLocation().getPath();
		location = URLDecoder.decode(location, "UTF-8").replaceAll("\\\\", "/");
		final String app = Textube.class.getCanonicalName();
		final String flags = "-Xmx512m -Dsun.java2d.d3d=false";
		boolean sh = true;
		final char q = '"', s = ' ';
		final StringBuilder param = new StringBuilder(64);
		
		switch (Configuration.getOS()) {
		case WINDOWS:
			sh = false;
			param.append("javaw");
			param.append(s);
			param.append(flags);
			break;
		case MAC:
			param.append("java");
			param.append(s);
			param.append(flags);
			param.append(s);
			param.append("-Xdock:name=");
			param.append(q);
			param.append(Configuration.NAME);
			param.append(q);
			param.append(s);
			param.append("-Xdock:icon=");
			param.append(q);
			param.append(Configuration.Paths.Resources.Images.ICON_32);
			param.append(q);
			param.append(s);
			param.append("-Dapple.laf.useScreenMenuBar=true");
			break;
		default:
			param.append("java");
			param.append(s);
			param.append(flags);
			break;
		}

		param.append(s);
		param.append("-classpath");
		param.append(s);
		param.append(q);
		param.append(location);
		param.append(q);
		
		for (String path : Configuration.Paths.getLibraryPaths()) {
			path = URLDecoder.decode(path, "UTF-8").replaceAll("\\\\", "/");
			param.append(File.pathSeparatorChar);
			param.append(q);
			param.append(path);
			param.append(q);
		}
		
		param.append(s);
		param.append(app);

		for (final String arg : args) {
			param.append(s);
			param.append(q);
			param.append(arg);
			param.append(q);
		}

		final Runtime run = Runtime.getRuntime();
		if (sh) {
			run.exec(new String[]{"/bin/sh", "-c", param.toString()});
		} else {
			run.exec(param.toString());
		}
	}

}
